
# q-expansion of newform 3234.2.a.bi, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3234
weight = 2
poly_data = [-3, -6, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[0, 2, 0], 1], [[-4, -1, 1], 1]]

hecke_ring_character_values = None
aps_data = [[1, 0, 0], [1, 0, 0], [0, 0, 1], [0, 0, 0], [1, 0, 0], [0, 0, 2], [1, 0, 0], [-1, 1, -1], [3, 1, 0], [3, -1, -1], [2, -2, -2], [3, 1, -1], [4, -1, -1], [3, 1, -1], [-5, 1, 0], [0, 0, 2], [3, 1, -3], [-2, -2, -3], [2, -1, -1], [1, -1, -1], [0, 2, 0], [4, 0, 3], [-2, 1, -3], [-12, 0, 2], [3, 2, 2], [-5, -1, 5], [-4, 2, -2], [8, 1, 5], [2, 2, 3], [-4, 2, 2], [3, 1, -2], [-6, -2, -6], [0, 0, 2], [9, -3, -3], [9, -1, 1], [1, -1, -4], [-3, -3, 3], [-4, -1, -5], [4, -2, 4], [-14, -2, -2], [-9, -1, -3], [10, -2, -4], [8, 0, 4], [-2, 2, -4], [7, 3, 1], [-12, 2, -2], [4, -4, -2], [0, 2, 4], [4, 1, 1], [-2, 2, 0], [7, 2, -2], [-4, -4, -2], [0, 2, -6], [-7, 1, -5], [-16, 0, 2], [-10, -4, 6], [-10, -2, -5], [8, 0, 4], [8, 0, 6], [-9, 2, 2], [4, -4, -6], [-7, 3, 3], [0, 2, -2], [-3, -1, 4], [25, 1, 1], [-4, -4, -5], [0, -5, -5], [-2, -2, 8], [-14, -1, -5], [12, 0, 5], [-16, -2, -4], [-10, 0, -8], [20, 0, 6], [2, 2, -5], [0, 1, 5], [5, -5, -1], [-4, 0, 1], [13, -3, 1], [-2, 2, 4], [-18, 6, 4], [-1, -1, -7], [-17, 3, 1], [4, 4, -10], [9, -2, -6], [-1, -3, 4], [-19, 1, 5], [0, -4, -6], [-10, -4, 4], [-23, 5, 1], [-20, 2, 0], [21, 3, -1], [-2, 0, 6], [-28, -2, 4], [2, 5, -7], [2, -2, 10], [2, -2, 4], [-4, 4, -2], [-2, 0, -8], [20, -4, -6], [16, -4, 1], [5, 3, 1], [-23, -3, -1], [0, 0, 0], [15, 3, 7], [9, 1, 1], [12, -3, -7], [-32, -2, -4], [-11, 5, 5], [-4, -4, -1], [22, -2, -2], [12, -4, 1], [0, -4, -5], [-2, -6, -4], [16, 3, -9], [8, 6, 8], [-10, -2, 2], [-10, 6, 6], [12, 4, -5], [0, 4, 3], [36, 1, 5], [-11, 1, -5], [4, 6, 2], [-25, -3, 3], [-5, -7, -7], [0, -3, -7], [5, 3, 5], [-15, -1, -1], [-9, 1, -4], [6, 4, -4], [6, 2, -1], [-16, 6, 6], [-34, -2, 0], [-10, 2, -8], [7, 3, 7], [20, 1, 1], [18, -10, -4], [-26, -2, -9], [7, 9, -1], [-2, -2, -9], [-16, 1, 1], [-40, 2, 2], [10, -2, -6], [-6, -6, 0], [16, -5, -5], [47, 1, -1], [-8, -8, 1], [12, 0, 3], [5, 4, 0], [-2, 1, -3], [-28, -4, -1], [0, -4, -7], [0, 2, 2], [8, -9, -9], [12, -6, 0], [-6, -3, 9], [15, 1, -14], [-19, -5, 6], [10, -6, -8], [-8, 6, 6], [-33, 3, 3], [-11, 5, 9], [20, 5, -7], [31, 1, -4], [-12, -6, -2], [-12, 8, 4], [-33, -3, 4], [8, 0, 10], [40, 0, 2], [20, 0, 6], [2, 6, 4], [-8, -5, 7], [6, 2, 20], [2, -12, -4], [20, 8, -10], [-28, -6, 12], [-2, -2, 16], [11, -1, 7], [26, 2, 8], [-12, 8, 13], [-16, -4, -10], [-8, -4, 6], [8, -1, 11], [15, 5, 15], [1, -2, -6], [7, 9, 5], [-20, 0, 10], [4, -4, -9], [-22, -1, 3], [36, 4, 4], [-10, -2, 8], [29, 5, 1], [-4, 3, 15], [-10, -1, 7], [-54, 2, 0], [18, 2, 2], [-38, 0, 0], [40, -2, 6], [17, 3, 3], [2, 0, -16], [38, 6, -2], [2, -8, -12], [-11, -5, 7], [14, 6, -4], [-16, -4, -8], [-1, 11, 15], [-12, 0, 10], [-12, 8, 4], [-4, -5, -17], [16, 3, 19], [42, -2, -2], [22, 0, 8], [20, 8, -1], [-20, -2, -4], [8, 6, 0], [32, -4, 2], [21, 1, 13], [-8, 4, -4], [12, -1, 15], [-24, 4, 1], [6, -10, 0], [22, 6, 8], [-11, -5, -8], [14, -6, 12], [38, -2, 2], [14, -5, -9], [12, -8, -7], [18, 4, -2], [-5, -11, -8], [21, 7, -6], [36, 4, 0], [23, -3, -3], [15, 7, -7], [17, -1, -5], [25, -2, -2], [-8, -5, -1], [-8, 6, -4], [5, -3, 21], [6, 4, 10], [-4, 4, 2], [20, 4, -1], [19, 5, 13], [-8, 8, 8], [-50, 2, 2], [5, 1, 1], [49, 5, -7], [0, 4, -2], [-20, -10, 18], [9, -3, 9], [-21, 7, 13], [32, -4, 8], [-4, 0, -6], [-23, -3, -7], [31, 9, -6], [20, 9, -3], [-14, 0, 16], [-2, 6, -2], [-2, 4, 18], [31, 7, 11], [2, -6, -4], [24, -6, -4], [-4, -8, -17], [-2, -6, -10], [-12, -4, -3], [34, 2, -1], [-14, -4, 10], [-10, -2, 14], [-10, 14, 5], [18, 2, 6], [-49, 1, 7], [22, 8, 2], [-11, -11, -5], [-30, 5, 1], [38, 0, 8], [23, -3, 0], [22, 8, 10], [38, -2, 8], [-11, -9, 1], [26, 2, 9], [-30, -4, 10], [-20, 7, -1], [10, 12, -4], [-12, -2, -18], [-2, 6, -2], [-22, 2, 15], [-21, -5, 9], [33, -9, 0], [-2, -4, 4], [43, -5, -3], [4, -10, -2], [-7, 1, 17], [6, 10, -9], [21, 3, 25], [8, 5, 13], [24, -7, 1], [10, 2, -17], [42, 2, -12], [-40, 6, -2], [-6, -10, 3], [0, 5, 1], [-5, -13, -13], [-18, -8, -18], [29, -3, 17], [-9, 1, -3], [-33, -1, -9], [36, -1, 3], [56, 3, -5], [-4, 4, -16], [-1, -1, -15], [-29, 13, 13], [14, 2, 11], [6, 4, 4], [-30, 10, 4], [7, 4, 0], [2, 13, 21], [-8, -14, -8], [-26, 2, 12], [13, 5, 5], [38, -2, 14], [11, -4, 0], [-21, -5, 3], [-27, 5, -3], [-7, -14, -6], [-7, -15, 3], [0, -8, -12], [-23, 9, -11], [-14, -4, -4], [-34, -3, 5], [-14, 1, -3], [-30, 10, -2], [18, -14, -6], [36, 0, 3], [20, 8, 2], [-43, -5, 7], [2, -7, -19], [-20, 0, -12], [4, 0, 22], [-23, 7, 7], [45, 5, -3], [7, 3, -21], [5, 3, -21], [-10, 6, 6], [28, 8, -10], [33, -1, -5], [20, 2, 6], [-8, 12, -2], [8, 0, 12], [37, 9, 21], [18, 6, 26], [-46, -6, -6], [6, -6, -1], [1, 3, -3], [25, -10, -2], [-26, -6, 19], [-45, -7, -18], [22, -6, 0], [3, 7, 19], [-4, -8, -7], [42, 1, 13], [-9, 11, 3], [12, 4, -5], [-10, 6, -4], [-29, -6, -6], [-16, -4, -28], [-6, -1, -9], [64, 0, 4], [-39, -8, 12], [9, 5, -19], [-12, -8, -8], [48, 2, 6], [23, -7, -1], [-18, -13, -1], [-54, -8, 12], [26, 6, 4], [-4, -12, 0], [-33, 9, 1], [53, -3, 11], [0, 4, 10], [8, 14, 4], [8, 1, 17], [-24, 10, 8], [-32, -8, 13], [28, -16, -16], [-12, -12, -19], [-13, -5, 27], [-71, -1, -5], [4, -4, -5], [-22, 12, -4], [51, 5, 3], [15, 9, -23], [-5, 5, -10], [-61, 0, 12], [46, -2, 13], [-19, 5, 17], [-12, -3, 1], [-29, -3, 10], [24, -2, 20], [-47, 11, 11], [4, 4, -6], [10, 6, -22], [-9, -7, -21], [-62, -6, -12], [-32, -6, 4], [0, 0, 4], [-14, 8, 10], [-11, 7, 25], [44, -4, -5], [-25, -9, 11], [43, 0, 0], [-8, 19, 15], [3, -11, 7], [-34, 4, 12], [13, -11, -21], [-63, 7, 7], [7, -3, 13], [28, 8, -20], [6, 10, -7], [-2, -4, -8], [38, -4, -8], [-2, 12, 14], [30, -6, -8], [-42, -6, -11], [16, 12, -9], [-11, -5, -17], [10, 6, 10], [-11, 5, -11], [40, -8, -3], [-36, -2, -4], [-31, -3, -7], [-6, -9, -17], [-18, -12, 2]]
